/* This file is part of SLB
   Copyright (C) 2011 Sergey Poznyakoff

   SLB is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3, or (at your option)
   any later version.

   SLB is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with SLB.  If not, see <http://www.gnu.org/licenses/>. */

#include "slb.h"

struct slb_oidref
{
	struct slb_varinstance const *inst;
};

static unsigned
oidref_hash(void *entry, unsigned long hash_size)
{
	struct slb_oidref *ref = entry;
	int i;
	unsigned hash = 0;

	for (i = 0; i < ref->inst->vi_oidlen; i++) {
		hash <<= 1;
		hash ^= ref->inst->vi_oid[i];
	}
	return hash % hash_size;
}

static int
oidref_cmp(const void *a, const void *b)
{
	const struct slb_oidref *aref = a;
	const struct slb_oidref *bref = b;

	if (aref->inst == bref->inst)
		return 0;
	return snmp_oid_compare(aref->inst->vi_oid, aref->inst->vi_oidlen,
				bref->inst->vi_oid, bref->inst->vi_oidlen);
}

static int
oidref_copy(void *a, void *b)
{
	struct slb_oidref *aref = a;
	struct slb_oidref *bref = b;

	aref->inst = bref->inst;
	return 0;
}

void
oidref_free(void *p)
{
	free(p);
}

static int
_create_ref(void *sym, void *data)
{
	struct slb_varinstance *vinst = sym;
	struct symtab *oidtab = data;
	struct slb_oidref key, *ref;
	int install = 1;
	
	key.inst = vinst;
	return symtab_lookup_or_install((void**)&ref, oidtab, &key, &install);
}
	
struct symtab *
create_oidtab(struct symtab *vit)
{
	struct symtab *oidtab;

	oidtab = symtab_create(sizeof(struct slb_oidref),
			       oidref_hash, oidref_cmp, oidref_copy, NULL,
			       oidref_free);
	if (!oidtab)
		xalloc_die();

	symtab_enumerate(vit, _create_ref, oidtab);
	return oidtab;
}

struct slb_varinstance *
oidtab_lookup(struct symtab *oidtab, oid *name, size_t name_length)
{
	struct slb_varinstance key_inst;
	struct slb_oidref key, *ref;
	int rc;

	memcpy(key_inst.vi_oid, name, sizeof(name[0]) * name_length);
	key_inst.vi_oidlen = name_length;
	key.inst = &key_inst;
	rc = symtab_lookup_or_install((void**)&ref, oidtab, &key, NULL);
	if (rc == ENOENT)
		return NULL;
	if (rc) {
		logmsg(LOG_CRIT, _("lookup in oidtab failed: %s"),
		       symtab_strerror(rc));
		exit(EX_SOFTWARE);
	}
	return (struct slb_varinstance*)ref->inst;
}

		
